mirror of https://lore.kernel.org/lkml/
 help / color / mirror / Atom feed
* RE: Fixing /proc/kcore
@ 2002-10-28 19:03 Luck, Tony
  2002-10-28 19:10 ` Andi Kleen
  0 siblings, 1 reply; 4+ messages in thread
From: Luck, Tony @ 2002-10-28 19:03 UTC (permalink / raw)
  To: Andi Kleen; +Cc: IA64 Linux Mail Group, Linux Kernel Mailing List

Andi Kleen wrote:
> On Fri, Oct 25, 2002 at 02:14:44PM -0700, Luck, Tony wrote:
> > Putting everything into the vmlist looks to be a good idea.  Perhaps
> > there should be an entry for the "direct addresses" too?
> 
> Yes that would make sense.
> 
> > 
> > So what does:
> > 
> > 	# objdump -p /proc/kcore
> > 
> > look like for you?
> 
> Very messy because of the negative addresses and still some sign problems
> in binutils :-)

Oops, I didn't mean to take this discussion off the mailing list.

What about a combined approach ... architecture dependent code
should add all the interesting stuff to the vmlist, so kcore just
needs to walk the list to cover everything.  We could also keep the
KCORE_BASE concept from my patch, but turn it into a variable that
the architecture dependent code can set to some suitable offset to
keep all the offsets in /proc/kcore positive.  This will avoid
having to fixup binutils (at least until someone comes up with an
architecture where kernel space scatters across a wide enough range
that we can't keep the offsets positive).

-Tony

^ permalink raw reply	[flat|nested] 4+ messages in thread
* Fixing /proc/kcore
@ 2002-10-25 17:49 Luck, Tony
  0 siblings, 0 replies; 4+ messages in thread
From: Luck, Tony @ 2002-10-25 17:49 UTC (permalink / raw)
  To: 'Mario Smarduch', IA64 Linux Mail Group; +Cc: Linux Kernel Mailing List

[-- Attachment #1: Type: text/plain, Size: 1170 bytes --]

/proc/kcore is what you need, but it is broken on ia64 (and
has been since the dawn of time for access to region 5) because
it assumes that all kernel virtual addresses are above PAGE_OFFSET.
This isn't true on ia64, VMALLOC_START is smaller than PAGE_OFFSET.

Attached is a patch (applies to 2.4.19 and to 2.5.39) that fixes the
assumption.  After applying you'll be able to use:

	# gdb vmlinux /proc/kcore

and happily ask gdb to examine addresses in region 5.

-Tony Luck



-----Original Message-----
From: Mario Smarduch [mailto:cms063@email.mot.com]
Sent: Friday, October 25, 2002 7:36 AM
To: IA64 Linux Mail Group
Subject: [Linux-ia64] Debugger/Analysis tool for IA64 Kernel Reg 5


Hi,
    I'm wondering if there is a tool available (gdb or some crash
analysis
tool) that can be used to disassemble/dump region 5 pages? We recently
have ported LiS and OpenSS7 stacks to IA-64 and it was painful without
being able to debug the Reg 5 memory, but we'll still be doing more
work.
We currently just have a crude tool that gets the reg 7 address from a
reg 5 address and then we use gdb, this is pretty cumbersome.....

- Mario.


[-- Attachment #2: kcore.patch --]
[-- Type: application/octet-stream, Size: 2357 bytes --]

diff -ru ../../REF/linux-2.5.39-ia64-020928/fs/proc/kcore.c aegl-kcore/fs/proc/kcore.c
--- ../../REF/linux-2.5.39-ia64-020928/fs/proc/kcore.c	Fri Sep 27 14:48:35 2002
+++ aegl-kcore/fs/proc/kcore.c	Fri Oct 25 08:25:31 2002
@@ -99,6 +99,12 @@
 }
 #else /* CONFIG_KCORE_AOUT */
 
+#if VMALLOC_START < PAGE_OFFSET
+#define	KCORE_BASE	VMALLOC_START
+#else
+#define	KCORE_BASE	PAGE_OFFSET
+#endif
+
 #define roundup(x, y)  ((((x)+((y)-1))/(y))*(y))
 
 /* An ELF note in memory */
@@ -118,7 +124,7 @@
 	struct vm_struct *m;
 
 	*num_vma = 0;
-	size = ((size_t)high_memory - PAGE_OFFSET + PAGE_SIZE);
+	size = ((size_t)high_memory - KCORE_BASE + PAGE_SIZE);
 	if (!vmlist) {
 		*elf_buflen = PAGE_SIZE;
 		return (size);
@@ -126,15 +132,15 @@
 
 	for (m=vmlist; m; m=m->next) {
 		try = (size_t)m->addr + m->size;
-		if (try > size)
-			size = try;
+		if (try > KCORE_BASE + size)
+			size = try - KCORE_BASE;
 		*num_vma = *num_vma + 1;
 	}
 	*elf_buflen =	sizeof(struct elfhdr) + 
 			(*num_vma + 2)*sizeof(struct elf_phdr) + 
 			3 * sizeof(struct memelfnote);
 	*elf_buflen = PAGE_ALIGN(*elf_buflen);
-	return (size - PAGE_OFFSET + *elf_buflen);
+	return size + *elf_buflen;
 }
 
 
@@ -237,7 +243,7 @@
 	offset += sizeof(struct elf_phdr);
 	phdr->p_type	= PT_LOAD;
 	phdr->p_flags	= PF_R|PF_W|PF_X;
-	phdr->p_offset	= dataoff;
+	phdr->p_offset	= PAGE_OFFSET - KCORE_BASE + dataoff;
 	phdr->p_vaddr	= PAGE_OFFSET;
 	phdr->p_paddr	= __pa(PAGE_OFFSET);
 	phdr->p_filesz	= phdr->p_memsz = ((unsigned long)high_memory - PAGE_OFFSET);
@@ -254,7 +260,7 @@
 
 		phdr->p_type	= PT_LOAD;
 		phdr->p_flags	= PF_R|PF_W|PF_X;
-		phdr->p_offset	= (size_t)m->addr - PAGE_OFFSET + dataoff;
+		phdr->p_offset	= (size_t)m->addr - KCORE_BASE + dataoff;
 		phdr->p_vaddr	= (size_t)m->addr;
 		phdr->p_paddr	= __pa(m->addr);
 		phdr->p_filesz	= phdr->p_memsz	= m->size;
@@ -385,9 +391,9 @@
 	/*
 	 * Fill the remainder of the buffer from kernel VM space.
 	 * We said in the ELF header that the data which starts
 	 * at 'elf_buflen' is virtual address PAGE_OFFSET. --rmk
 	 */
-	start = PAGE_OFFSET + (*fpos - elf_buflen);
+	start = KCORE_BASE + (*fpos - elf_buflen);
 	if ((tsz = (PAGE_SIZE - (start & ~PAGE_MASK))) > buflen)
 		tsz = buflen;
 		

^ permalink raw reply	[flat|nested] 4+ messages in thread

end of thread, other threads:[~2002-10-28 19:07 UTC | newest]

Thread overview: 4+ messages (download: mbox.gz / follow: Atom feed)
-- links below jump to the message on this page --
     [not found] <39B5C4829263D411AA93009027AE9EBB1EF28F7B@fmsmsx35.fm.intel.com.suse.lists.linux.kernel>
@@ -385,9 +391,9 @@
 	 * Fill the remainder of the buffer from kernel VM space.
 	 * We said in the ELF header that the data which starts
-	 * at 'elf_buflen' is virtual address PAGE_OFFSET. --rmk
+	 * at 'elf_buflen' is virtual address KCORE_BASE. --rmk
 	 */
2002-10-25 18:00 ` Fixing /proc/kcore Andi Kleen
2002-10-28 19:03 Luck, Tony
2002-10-28 19:10 ` Andi Kleen
  -- strict thread matches above, loose matches on Subject: below --
2002-10-25 17:49 Luck, Tony

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox

Powered by JetHome